import collections


class TransformedDict(collections.MutableMapping):

    """A dictionary that applies an arbitrary key-altering
       function before accessing the keys.

       from: http://stackoverflow.com/questions/3387691/python-how-to-perfectly-override-a-dict
       """

    def __init__(self, *args, **kwargs):
        self.store = dict()
        self.update(dict(*args, **kwargs))  # use the free update to set keys

    def __getitem__(self, key):
        return self.store[self.__keytransform__(key)]

    def __setitem__(self, key, value):
        self.store[self.__keytransform__(key)] = value

    def __delitem__(self, key):
        del self.store[self.__keytransform__(key)]

    def __iter__(self):
        return iter(self.store)

    def __len__(self):
        return len(self.store)

    def __keytransform__(self, key):
        return key
